@@ -1,34 +1,39 @@
 """The :class:`RunDirectory` class is a convenience class to  access and apply
-the tint tracking algoritm."""
+the tint tracking algorithm."""
 from __future__ import annotations
-from datetime import datetime
-import hashlib
-from pathlib import Path
-from typing import Any, Optional, Union, Iterator
-import warnings
 
+import hashlib
+import warnings
+from datetime import datetime
+from pathlib import Path
+from typing import Any, Iterator, Optional, Union
+
+import geopandas as gpd
+import pandas as pd
+import pyproj
+import xarray as xr
+from cartopy.crs import AzimuthalEquidistant, Projection
 from cartopy.mpl.geoaxes import GeoAxesSubplot
 from matplotlib import pyplot as plt
-import pandas as pd
+from shapely import wkt
 from tqdm.auto import tqdm
-import xarray as xr
 
-from .helpers import get_grids, convert_to_cftime, MetaData
-from .tracks import Cell_tracks
-from .visualization import full_domain, plot_traj
 from .config import set as set_config
+from .helpers import MetaData, convert_to_cftime, get_grids
+from .tracks import Cell_tracks
 from .types import GridType
+from .visualization import full_domain, plot_traj
 
 __all__ = ["RunDirectory"]
 
 
 class RunDirectory(Cell_tracks):
     """
-    Create an :class:`RunDirecotry` object from a given xarray dataset.
+    Create an :class:`RunDirectory` object from a given xarray dataset.
 
-    The :class:`RunDirectory` object gathers all necessary information on the
-    data that is stored in the run directory. Once loaded the most
-    important meta data will be stored in the run directory for faster
+    The :class:`RunDirectory` object gathers all necessary information of
+    data that is stored in a ``xarray`` dataset. Once loaded the most
+    important metadata will be stored in the run directory for faster
     access the second time.
 
     Parameters
@@ -39,11 +44,10 @@ class RunDirectory(Cell_tracks):
         Name of the variable that is tracked
     x_coord: str, default: lon
         Name of the X coordinate array/vector
-    y_coord: st, default: lat
+    y_coord: str, default: lat
         Name of the Y coordinate array/vector
     time_coord: str, default: time
         Name of the time variable.
-    start:
 
     Example
     -------
@@ -74,14 +78,15 @@ class RunDirectory(Cell_tracks):
         time_coord: str = "time",
         start: Optional[Union[str, datetime, pd.Timestamp]] = None,
         end: Optional[Union[str, datetime, pd.Timestamp]] = None,
+        crs: Union[str, pyproj.CRS, Projection] = "epsg:4326",
         **kwargs: Any,
     ) -> RunDirectory:
         """
-        Create an :class:`RunDirectory` object from given input file(s)/ directory.
+        Create a :class:`RunDirectory` object from input file(s)/directory.
 
-        The :class:`RunDirectory` object gathers all necessary information on the
-        data that is stored in the run directory. Once loaded the most
-        important meta data will be stored in the run directory for faster
+        The :class:`RunDirectory` object gathers all necessary information of
+        the data that is stored in the run directory. Once loaded the most
+        important metadata will be stored in the run directory for faster
         access the second time.
 
         Parameters
@@ -91,17 +96,22 @@ class RunDirectory(Cell_tracks):
         var_name: str
             Name of the variable that is tracked
         start: str, pandas.Timestamp, datetime.datetime (default: None)
-            first time step that is considered, if none given the first
-            of the dataset is considered.
+            first time step that is considered, if None given the first
+            time step in the data is considered.
         end: str, pandas.Timestamp, datetime.datetime (default: None)
-            last time step that is considered, if none given the last
-            of the dataset is considered.
+            last time step that is considered, if None given the last
+            time step in the data is considered.
         x_coord: str (default: lon)
-            The name of the longitude vector/array
+            The name of the longitude vector/array, can be 1D or 2D
         x_coord: str (default: lat)
-            The name of the latitude vector/array
+            The name of the latitude vector/array, can be 1D or 2D
         time_coord: str, default: time
             The name of the time variable
+        crs: str, pyproj.Proj, cartopy.crs.Projection (default: "epsg:4326")
+            pyproj/cartopy projection object or string defining the Coordinate
+            Reference System (CRS). If "aeqd", CRS will be computed as
+            AzimuthalEquidistant Projection from the geodetic radar site
+            coordinates "longitude" and "latitude".
         kwargs:
             Additional keyword arguments that are passed to open the dataset
             with xarray
@@ -132,12 +142,21 @@ class RunDirectory(Cell_tracks):
         _dset = xr.open_mfdataset(_files, **kwargs)
         start_time = start or _dset[time_coord].isel({time_coord: 0})
         end_time = end or _dset[time_coord].isel({time_coord: -1})
+
+        # create AEQD CRS
+        if crs == "aeqd":
+            crs = AzimuthalEquidistant(
+                central_latitude=_dset.latitude.mean().values.item(),
+                central_longitude=_dset.longitude.mean().values.item(),
+            )
+
         return cls(
             _dset.sel({time_coord: slice(start_time, end_time)}),
             var_name,
             x_coord=x_coord,
             y_coord=y_coord,
             time_coord=time_coord,
+            crs=crs,
             _files=_files,
         )
 
@@ -149,6 +168,7 @@ class RunDirectory(Cell_tracks):
         time_coord: str = "time",
         x_coord: str = "lon",
         y_coord: str = "lat",
+        crs: Union[str, pyproj.CRS, Projection] = "epsg:4326",
         _files: Union[list[str], str] = "",
     ) -> None:
         if isinstance(dataset, xr.DataArray):
@@ -162,6 +182,17 @@ class RunDirectory(Cell_tracks):
         self.time = self.data[time_coord]
         self.start = convert_to_cftime(self.time.values[0])
         self.end = convert_to_cftime(self.time.values[-1])
+        # transform crs to wkt in any case
+        if isinstance(crs, Projection):
+            self.crs = pyproj.CRS(crs.proj4_init).to_wkt()
+        elif isinstance(crs, pyproj.CRS):
+            self.crs = crs.to_wkt()
+        elif isinstance(crs, str):
+            self.crs = pyproj.CRS(crs).to_wkt()
+        else:
+            raise TypeError(
+                "crs parameter must be a string or pyproj/cartopy " "projection"
+            )
         self._metadata_reader = MetaData(
             self.data,
             self.var_name,
@@ -176,9 +207,11 @@ class RunDirectory(Cell_tracks):
         flush: bool = True,
         **tracking_parameters: float,
     ) -> int:
-        """Obtains tracks given a list of data arrays. This is the
-            primary method of the tracks class. This method makes use of all of
-            the functions and helper classes defined above.
+        """Apply the ``tint`` tracking algorithm.
+
+        This is the primary method of the :class:`RunDirectory` class. This
+        method applies the tracking algorithm to the data and saves
+        the tracked cells to a pandas ``DataFrame`` for analysis.
 
         Parameters
         ----------
@@ -187,9 +220,10 @@ class RunDirectory(Cell_tracks):
         leave_bar: bool, default: True
             Leave the progress bar after tracking is finished
         flush: bool, default: True
-            Flush old tracking data.
+            Flush old tracking data. If false, the tracks will be added
+            to the existing ::method::`tracks` DataFrame.
         **tracking_parameters: float
-            Overwrite the tint tracking parameters with this values for this
+            Overwrite the tint tracking parameters with these values for this
             specific tracking. The defaults will be restored afterwards.
             See :py:mod:`tintx.config` for details
 
@@ -224,7 +258,10 @@ class RunDirectory(Cell_tracks):
         parameters = self._parameters.get(self._track_hash(), {})
         parameters.update(tracking_parameters)
         with tqdm(
-            self.grids, total=self.time.size - 1, desc="Tracking", leave=leave_bar
+            self.grids,
+            total=self.time.size - 1,
+            desc="Tracking",
+            leave=leave_bar,
         ) as pbar:
             with set_config(**parameters) as cfg:
                 num_tracks = self._get_tracks(self.grids, pbar, centre)
@@ -245,8 +282,14 @@ class RunDirectory(Cell_tracks):
 
     @property
     def tracks(self) -> pd.DataFrame:
-        """A pandas.DataFrame representation of the tracked cells."""
-        return self._tracks
+        """Pandas ``DataFrame`` representation of the tracked cells."""
+        # todo: cache GeoDataFrame like the normal DataFrame
+        # only convert into GeoDataFrame if geometry-column is available
+        # for backwards compatibility reasons
+        if "geometry" in self._tracks.columns:
+            return gpd.GeoDataFrame(self._tracks.copy(), crs=pyproj.CRS(self.crs))
+        else:
+            return self._tracks
 
     def save_tracks(self, output: Union[str, Path]) -> None:
         """Save tracked data to hdf5 table.
@@ -285,7 +328,13 @@ class RunDirectory(Cell_tracks):
                 warnings.simplefilter(
                     action="ignore", category=pd.errors.PerformanceWarning
                 )
-                hdf5.put("tintx_tracks", self.tracks)
+                tracks = self._tracks.copy()
+                # only save track geometries and crs as wkt strings if geometry-column
+                # is available for backwards compatibility reasons
+                if "geometry" in tracks.columns:
+                    tracks["geometry"] = self.tracks.geometry.map(str)
+                    metadata["crs"] = self.crs
+                hdf5.put("tintx_tracks", tracks)
                 self._metadata_reader.save(hdf5)
             table = hdf5.get_storer("tintx_tracks")
             table.attrs.track_meta = metadata
@@ -295,7 +344,7 @@ class RunDirectory(Cell_tracks):
     def from_dataframe(
         cls, track_file: Union[str, Path], dataset: Optional[xr.Dataset] = None
     ) -> RunDirectory:
-        """Create an instance of the RunDirectory class from tintx tracks.
+        """Create an instance of the :class:`RunDirectory` class from tintx tracks.
 
         Parameters
         ----------
@@ -307,10 +356,9 @@ class RunDirectory(Cell_tracks):
             Note: this only works if the previous instance of the tracking
             class was instanciated with the :class:`from_files` method.
 
-        Raises
-        ------
-        ValueError:
-            If the dataset can't be retrieved.
+        .. note::
+        If the dataset can't be opened an empty data set will be created
+        instead.
 
         Example
         -------
@@ -368,13 +416,19 @@ class RunDirectory(Cell_tracks):
             )
             cls_instance = cls(coord_dataset, var_name, **metadata)
         cls_instance._metadata[cls_instance._track_hash()] = parameters
+        # apply geometry conversion only if geometry-column is available
+        # for backwards compatibility reasons
+        if "geometry" in tracks.columns:
+            tracks["geometry"] = tracks["geometry"].apply(wkt.loads)
+            # add crs as wkt
+            cls_instance.crs = metadata["crs"]
         cls_instance.reset_tracks(tracks)
         return cls_instance
 
     def get_parameters(
         self, tracks: Optional[pd.DataFrame] = None
     ) -> dict[str, float]:
-        """Get the parameters of given object tracks.
+        """Get the parameters of given cell tracks.
 
         Parameters
         ----------
@@ -384,12 +438,11 @@ class RunDirectory(Cell_tracks):
 
         Returns
         -------
-        dict: dictionary holding the parameter information for the
-              given track object.
+        dict: dictionary holding the tuning parameters for the given tracks.
 
         Raises
         ------
-        ValueError: if no parameters matching the input tracks ``DataFrame```
+        ValueError: if no parameters matching the input tracks ``DataFrame``
                     could be found.
 
         Example
@@ -414,7 +467,6 @@ class RunDirectory(Cell_tracks):
             from tintx import RunDirectory
             run = RunDirectory.from_dataframe("/tmp/output.hdf5")
             parameters = run.get_parameters()
-            print(parameters)
         """
 
         try:
@@ -501,8 +553,7 @@ class RunDirectory(Cell_tracks):
         vmax : float, default: 15.0
             Maximum values for the colormap.
         isolated_only: bool, default: False
-            If true, only annotates uids for isolated objects. Only used in 'full'
-            style.
+            If true, only annotates uids for isolated objects.
         cmap: str, default: Blues
             Colormap used for plotting the tracked fields.
         alt: float, default: None
@@ -511,7 +562,7 @@ class RunDirectory(Cell_tracks):
             Plot traces of animated cells
         dt: float, default: 0
             Time shift in hours that is applied to the data, this can be
-            useful if data time data in in utc but should be displayed in
+            useful if time data is in utc but should be displayed in
             another time zone.
         fps: int, default: 5
             Frames per second for output.
@@ -529,19 +580,6 @@ class RunDirectory(Cell_tracks):
 
         Example
         -------
-        .. execute_code::
-            :hide_code:
-            :hide_headers:
-
-            from pathlib import Path
-            if not Path("/tmp/output.hdf5").is_file():
-                from tintx import RunDirectory
-                run = RunDirectory.from_files(
-                    os.path.join(os.environ["FILE_PATH"], "CPOL*.nc"),
-                    "radar_estimated_rain_rate", x_coord="x", y_coord="y"
-                )
-                run.get_tracks(min_size=4, field_thresh=0.1)
-                run.save_tracks("/tmp/output.hdf5")
 
         .. execute_code::
             :hide_headers:
@@ -574,7 +612,7 @@ class RunDirectory(Cell_tracks):
         self,
         label: bool = False,
         ax: Optional[GeoAxesSubplot] = None,
-        uids: list[str] = None,
+        uids: Optional[list[str]] = None,
         mintrace: int = 2,
         size: int = 50,
         thresh: float = -1.0,
@@ -582,16 +620,16 @@ class RunDirectory(Cell_tracks):
         plot_style: Optional[dict[str, Union[float, int, str]]] = None,
     ) -> GeoAxesSubplot:
         """
-        Plot traces of trajectories for each particle.
+        Plot traces of trajectories for each cell track.
 
-        This code is a fork of plot_traj method in the plot module from the
+        This code is a fork of ``plot_traj`` method in the plot module from the
         trackpy project see http://soft-matter.github.io/trackpy for more
         details
 
         Parameters
         ----------
         label : boolean, default: False
-            Set to True to write particle ID numbers next to trajectories.
+            Set to True to write cell uids next to trajectories.
         cmap : colormap, Default matplotlib.colormap.winter
             Colormap used to color different tracks
         ax: cartopy.mpl.geoaxes.GeoAxesSubplot, default: None
